#!/usr/bin/env bash

# This script is hereby placed in the Public Domain
#
# Produce a raw textual description of the finite-state automata calculated
# by FDR for any CSP process
#
# Requires: FDR to compile the CSP to a state machine
#
# Warning: This code is an awful hack and ought to be cleaned up properly.
#          My apologies in advance.
#
# setting CSP2TEXTDESC_NORMALIZE to a non-empty string value will cause
#         the ISM to be normalized after it is compiled

if [ -z $FDRHOME ]; then
    FDRHOME="/usr/local/fdr"
    echo "FDRHOME not set. Presuming: FDRHOME=$FDRHOME"
fi

FDR=$FDRHOME/bin/fdr2tix
FDRFLAGS="-insecure -nowindow"

# check for needed dependencies
if [ ! -f "$FDR" ]; then
    echo "Error: FDRHOME must be set correctly."
    exit 1;
fi


export FDRHOME=$FDRHOME

INPUTFILE=$1
PROCESS=$2
OUTPUTFILE=$3

INPUTDIR=$( dirname $INPUTFILE )
INPUTBASENAME=$( basename $INPUTFILE )

if [ ! -z "$CSP2TEXTDESC_NORMALIZE" ]; then
    # the 'compress' command doesn't allow us to specify the name of the
    # resulting compressed ISM. Hence, we assume it will be allocated to
    # 'ism__1'. Hopefully, future FDR releases won't break this assumption.
    NORMALIZE_COMMAND="myism compress normal"
    ISMNAME=ism__1
    echo "(Process will be normalized.)"
else
    ISMNAME=myism
fi

echo "Running FDR to produce raw description..."
<<EOF $FDR $FDRFLAGS > $OUTPUTFILE
source $FDRHOME/lib/fdr/fdrDirect.tcl
session mysession
mysession load $INPUTDIR $INPUTBASENAME
mysession compile [mysession evaluate "$PROCESS"] myism
$NORMALIZE_COMMAND
puts [$ISMNAME root]
puts [$ISMNAME transitions]
puts [$ISMNAME numeric_alphabet]
puts [$ISMNAME alphabet]
puts [$ISMNAME acceptances]
puts [$ISMNAME divergences]
EOF

# prepend the process name
# we do this rather than use 'puts [$ISMNAME name]' in FDR since for some
# processes, FDR doesn't have a name associated with the ISM for some reason
mv $OUTPUTFILE $OUTPUTFILE.old
echo "$PROCESS" > $OUTPUTFILE.new
cat $OUTPUTFILE.new $OUTPUTFILE.old > $OUTPUTFILE

rm -f $OUTPUTFILE.old $OUTPUTFILE.new

# check that FDR worked. If so, it should have produced 5 lines of output
NUMLINES=$( wc -l $OUTPUTFILE )
if [ -z "$( echo $NUMLINES | grep '7 ' )" ]; then
    echo "Error running FDR"
    exit 1;
fi

# everything worked OK
exit 0;
